@@ -2,7 +2,7 @@ use std::fmt;
 use std::borrow::Cow;
 use std::error::Error as StdError;
 
-///Struct which containing the current position
+///Struct which represents the positions in the source file
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Ord, PartialOrd)]
 pub struct SourcePosition {
     ///Current line of the input
@@ -25,17 +25,36 @@ impl SourcePosition {
 }
 
 ///Enum used to store information about an error that has occured
-#[derive(Clone, PartialEq, Debug)]
+#[derive(Debug)]
 pub enum Error {
     ///Error indicating an unexpected token has been encountered in the stream
     Unexpected(char),
     ///Error indicating that the parser expected something else
     Expected(Cow<'static, str>),
     ///Generic message
-    Message(Cow<'static, str>)
+    Message(Cow<'static, str>),
+    ///Variant for containing other types of errors
+    Other(Box<StdError>)
 }
 
-///Enum used to indicate if a stream has had any elements consumed
+impl PartialEq for Error {
+    fn eq(&self, other: &Error) -> bool {
+        match (self, other) {
+            (&Error::Unexpected(l), &Error::Unexpected(r)) => l == r,
+            (&Error::Expected(ref l), &Error::Expected(ref r)) => l == r,
+            (&Error::Message(ref l), &Error::Message(ref r)) => l == r,
+            _ => false
+        }
+    }
+}
+
+impl <E> From<E> for Error where E: StdError + 'static {
+    fn from(e: E) -> Error {
+        Error::Other(Box::new(e))
+    }
+}
+
+///Enum used to indicate if a parser consumed any items of the stream it was given as an input
 #[derive(Clone, PartialEq, Debug, Copy)]
 pub enum Consumed<T> {
     ///Constructor indicating that the parser has consumed elements
@@ -46,7 +65,7 @@ pub enum Consumed<T> {
 
 impl <T> Consumed<T> {
 
-    ///Returns true if the `self` is empty
+    ///Returns true if `self` is empty
     pub fn is_empty(&self) -> bool {
         match *self {
             Consumed::Empty(_) => true,
@@ -62,12 +81,12 @@ impl <T> Consumed<T> {
         }
     }
 
-    ///Converts the consumed state into the Consumed state
+    ///Converts `self` into the Consumed state
     pub fn as_consumed(self) -> Consumed<T> {
         Consumed::Consumed(self.into_inner())
     }
 
-    ///Converts the consumed state into the Empty state
+    ///Converts `self` into theEmpty state
     pub fn as_empty(self) -> Consumed<T> {
         Consumed::Empty(self.into_inner())
     }
@@ -82,6 +101,38 @@ impl <T> Consumed<T> {
     }
 
     ///Combines the Consumed flags from `self` and the result of `f`
+    ///
+    ///```
+    /// # extern crate parser_combinators as pc;
+    /// # use pc::*;
+    /// # use pc::primitives::State;
+    /// # fn main() {
+    /// //Parses a characther of string literal and handles the escaped characthers \\ and \" as \
+    /// //and " respectively
+    /// fn char(input: State<&str>) -> ParseResult<char, &str> {
+    ///     let (c, input) = try!(satisfy(|c| c != '"').parse_state(input));
+    ///     match c {
+    ///         //Since the `char` parser has already consumed some of the input `combine` is used
+    ///         //propagate the consumed state to the next part of the parser
+    ///         '\\' => input.combine(|input| {
+    ///             satisfy(|c| c == '"' || c == '\\')
+    ///                 .map(|c| {
+    ///                     match c {
+    ///                         '"' => '"',
+    ///                         '\\' => '\\',
+    ///                         c => c
+    ///                     }
+    ///                 })
+    ///                 .parse_state(input)
+    ///             }),
+    ///         _ => Ok((c, input))
+    ///     }
+    /// }
+    /// let result = many(parser(char))
+    ///     .parse(r#"abc\"\\"#);
+    /// assert_eq!(result, Ok((r#"abc"\"#.to_string(), "")));
+    /// }
+    ///```
     pub fn combine<F, U, I>(self, f: F) -> ParseResult<U, I>
         where F: FnOnce(T) -> ParseResult<U, I> {
         match self {
@@ -98,7 +149,7 @@ impl <T> Consumed<T> {
 }
 ///Struct which hold information about an error that occured at a specific position.
 ///Can hold multiple instances of `Error` if more that one error occured at the position.
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Debug, PartialEq)]
 pub struct ParseError {
     ///The position where the error occured
     pub position: SourcePosition,
@@ -187,7 +238,7 @@ impl fmt::Display for ParseError {
         }
         //If there are any generic messages we print them out last
         let messages = self.errors.iter()
-            .filter(|e| match **e { Error::Message(_) => true, _ => false } );
+            .filter(|e| match **e { Error::Message(_) | Error::Other(_) => true, _ => false } );
         for error in messages {
             try!(writeln!(f, "{}", error));
         }
@@ -205,6 +256,7 @@ impl fmt::Display for Error {
             Error::Unexpected(c) => write!(f, "Unexpected character '{}'", c),
             Error::Expected(ref s) => write!(f, "Expected {}", s),
             Error::Message(ref msg) => write!(f, "{}", msg),
+            Error::Other(ref err) => err.fmt(f)
         }
     }
 }
@@ -250,7 +302,8 @@ impl <I: Stream<Item=char>> State<I> {
 
 }
 
-///A type alias over the specific `Result` type used to indicated parser success/failure.
+///A type alias over the specific `Result` type used by parsers to indicate wether they were
+///successful or not.
 ///`O` is the type that is output on success
 ///`I` is the specific stream type used in the parser
 pub type ParseResult<O, I> = Result<(O, Consumed<State<I>>), Consumed<ParseError>>;
